@@ -41,6 +41,7 @@
 #include <osmocom/core/logging.h>
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/socket.h>
+#include <osmocom/core/sockaddr_str.h>
 
 #include <osmocom/netif/stream.h>
 #include <osmocom/netif/ipa.h>
@@ -58,6 +59,13 @@ LLIST_HEAD(osmo_ss7_instances);
 static int32_t next_rctx = 1;
 static int32_t next_l_rk_id = 1;
 
+const struct value_string mtp_unavail_cause_vals[] = {
+	{ MTP_UNAVAIL_C_UNKNOWN,		"unknown" },
+	{ MTP_UNAVAIL_C_UNEQUIP_REM_USER,	"unequipped-remote-user" },
+	{ MTP_UNAVAIL_C_INACC_REM_USER,		"inaccessible-remote-user" },
+	{ 0, NULL }
+};
+
 struct value_string osmo_ss7_as_traffic_mode_vals[] = {
 	{ OSMO_SS7_AS_TMOD_BCAST,	"broadcast" },
 	{ OSMO_SS7_AS_TMOD_LOADSHARE,	"loadshare" },
@@ -252,7 +260,7 @@ err:
 	return -EINVAL;
 }
 
-const char *_osmo_ss7_pointcode_print(char *buf, size_t len, const struct osmo_ss7_instance *inst, uint32_t pc)
+const char *osmo_ss7_pointcode_print_buf(char *buf, size_t len, const struct osmo_ss7_instance *inst, uint32_t pc)
 {
 	const struct osmo_ss7_pc_fmt *pc_fmt;
 	unsigned int num_comp_exp;
@@ -279,7 +287,7 @@ const char *_osmo_ss7_pointcode_print(char *buf, size_t len, const struct osmo_s
 const char *osmo_ss7_pointcode_print(const struct osmo_ss7_instance *inst, uint32_t pc)
 {
 	static char buf[MAX_PC_STR_LEN];
-	return _osmo_ss7_pointcode_print(buf, sizeof(buf), inst, pc);
+	return osmo_ss7_pointcode_print_buf(buf, sizeof(buf), inst, pc);
 }
 
 /* same as osmo_ss7_pointcode_print() but using a separate buffer, useful for multiple point codes in the
@@ -287,7 +295,7 @@ const char *osmo_ss7_pointcode_print(const struct osmo_ss7_instance *inst, uint3
 const char *osmo_ss7_pointcode_print2(const struct osmo_ss7_instance *inst, uint32_t pc)
 {
 	static char buf[MAX_PC_STR_LEN];
-	return _osmo_ss7_pointcode_print(buf, sizeof(buf), inst, pc);
+	return osmo_ss7_pointcode_print_buf(buf, sizeof(buf), inst, pc);
 }
 
 int osmo_ss7_pointcode_parse_mask_or_len(struct osmo_ss7_instance *inst, const char *in)
@@ -1055,8 +1063,8 @@ void osmo_ss7_as_destroy(struct osmo_ss7_as *as)
  *  \param[in] as Application Server in which to look for \ref asp
  *  \param[in] asp Application Server Process to look for in \ref as
  *  \returns true in case \ref asp is part of \ref as; false otherwise */
-bool osmo_ss7_as_has_asp(struct osmo_ss7_as *as,
-			 struct osmo_ss7_asp *asp)
+bool osmo_ss7_as_has_asp(const struct osmo_ss7_as *as,
+			 const struct osmo_ss7_asp *asp)
 {
 	unsigned int i;
 
@@ -1137,6 +1145,16 @@ int osmo_ss7_asp_peer_set_hosts(struct osmo_ss7_asp_peer *peer, void *talloc_ctx
 	return 0;
 }
 
+/* Is string formatted IPv4/v6 addr considered IN(6)ADDR_ANY? */
+static inline bool host_is_ip_anyaddr(const char *host, bool is_v6)
+{
+	/* NULL addr is resolved as 0.0.0.0 (IPv4) by getaddrinfo(), most
+	 * probably for backward-compatibility reasons.
+	 */
+	return is_v6 ? (host && !strcmp(host, "::"))
+		     : (!host || !strcmp(host, "0.0.0.0"));
+}
+
 /*! \brief Append (copy) address to a given ASP peer. Previous addresses are kept.
  *  \param[in] peer Application Server Process peer the address is appened to.
  *  \param[in] talloc_ctx talloc context used to allocate new address.
@@ -1146,28 +1164,28 @@ int osmo_ss7_asp_peer_add_host(struct osmo_ss7_asp_peer *peer, void *talloc_ctx,
 
 {
 	int i;
-	bool new_is_any = !host || !strcmp(host, "0.0.0.0");
-	bool iter_is_any;
+	bool new_is_v6 = osmo_ip_str_type(host) == AF_INET6;
+	bool new_is_any = host_is_ip_anyaddr(host, new_is_v6);
+	bool iter_is_v6;
 
 	if (peer->host_cnt >= ARRAY_SIZE(peer->host))
 		return -EINVAL;
 
-	/* Makes no sense to have INET_ANY many times, or INET_ANY together with
-	   specific addresses: */
+	/* Makes no sense to have INET(6)_ANY many times, or INET(6)_ANY
+	   together with specific addresses, be it of same or different
+	   IP version:*/
 	if (new_is_any && peer->host_cnt != 0)
 		return -EINVAL;
 
-	/* Makes no sense to add specific address to set if INET_ANY is
-	   already set: */
 	if (!new_is_any) {
+		/* Makes no sense to add specific address to set if INET(6)_ANY
+		   is already set, be it from same or different IP version: */
 		for (i = 0; i < peer->host_cnt; i++) {
-				iter_is_any = !peer->host[i] ||
-					      !strcmp(peer->host[i], "0.0.0.0");
-				if (iter_is_any)
+				iter_is_v6 = osmo_ip_str_type(peer->host[i]) == AF_INET6;
+				if (host_is_ip_anyaddr(peer->host[i], iter_is_v6))
 					return -EINVAL;
 		}
 	}
-
 	osmo_talloc_replace_string(talloc_ctx, &peer->host[peer->host_cnt], host);
 	peer->host_cnt++;
 	return 0;
@@ -1198,6 +1216,26 @@ static uint16_t get_in_port(struct sockaddr *sa)
 	}
 }
 
+/* Converts string representation of v4-mappend-on-v6 IP addr to a pure IPv4 address.
+ * Example: ::ffff:172.18.19.200 => 172.18.19.200
+ */
+static void chop_v4_mapped_on_v6_prefix(char* buf)
+{
+	char *last_colon;
+	size_t len;
+	char *first_dot = strchr(buf, '.');
+
+	if (!first_dot)
+		return; /* Not an IPv4-mappend-on-v6 string representation, nothing to do */
+	last_colon = strrchr(buf, ':');
+	if (!last_colon)
+		return; /* pure IPv4 address, nothing to do */
+
+	len = strlen(last_colon + 1);
+	memmove(buf, last_colon + 1, len);
+	buf[len] = '\0';
+}
+
 /*! \brief Find an ASP definition matching the local+remote IP/PORT of given fd
  *  \param[in] fd socket descriptor of given socket
  *  \returns SS7 ASP in case a matching one is found; NULL otherwise */
@@ -1205,33 +1243,45 @@ static struct osmo_ss7_asp *
 osmo_ss7_asp_find_by_socket_addr(int fd)
 {
 	struct osmo_ss7_instance *inst;
-	struct sockaddr sa_l, sa_r;
+	struct sockaddr_storage sa_l, sa_r;
 	socklen_t sa_len_l = sizeof(sa_l);
 	socklen_t sa_len_r = sizeof(sa_r);
 	char hostbuf_l[64], hostbuf_r[64];
 	uint16_t local_port, remote_port;
+	bool loc_is_v6, rem_is_v6;
 	int rc;
 	int i;
 
 	OSMO_ASSERT(ss7_initialized);
 	/* convert local and remote IP to string */
-	rc = getsockname(fd, &sa_l, &sa_len_l);
+	rc = getsockname(fd, (struct sockaddr*)&sa_l, &sa_len_l);
 	if (rc < 0)
 		return NULL;
-	rc = getnameinfo(&sa_l, sa_len_l, hostbuf_l, sizeof(hostbuf_l),
+	rc = getnameinfo((struct sockaddr*)&sa_l, sa_len_l,
+			 hostbuf_l, sizeof(hostbuf_l),
 			 NULL, 0, NI_NUMERICHOST);
 	if (rc < 0)
 		return NULL;
-	local_port = ntohs(get_in_port(&sa_l));
+	local_port = ntohs(get_in_port((struct sockaddr*)&sa_l));
 
-	rc = getpeername(fd, &sa_r, &sa_len_r);
+	rc = getpeername(fd, (struct sockaddr*)&sa_r, &sa_len_r);
 	if (rc < 0)
 		return NULL;
-	rc = getnameinfo(&sa_r, sa_len_r, hostbuf_r, sizeof(hostbuf_r),
+	rc = getnameinfo((struct sockaddr*)&sa_r, sa_len_r,
+			 hostbuf_r, sizeof(hostbuf_r),
 			 NULL, 0, NI_NUMERICHOST);
 	if (rc < 0)
 		return NULL;
-	remote_port = ntohs(get_in_port(&sa_r));
+	remote_port = ntohs(get_in_port((struct sockaddr*)&sa_r));
+
+	/* If multi-home is used with both IPv4 and IPv6, then the socket is
+	 * AF_INET6, and then returned IPv4 addresses are actually v6mapped ones.
+	 * We need to convert them to IPv4 before matching.
+	 */
+	chop_v4_mapped_on_v6_prefix(hostbuf_l);
+	chop_v4_mapped_on_v6_prefix(hostbuf_r);
+	loc_is_v6 = osmo_ip_str_type(hostbuf_l) == AF_INET6;
+	rem_is_v6 = osmo_ip_str_type(hostbuf_r) == AF_INET6;
 
 	/* check all instances for any ASP definition matching the
 	 * address combination of local/remote ip/port */
@@ -1244,8 +1294,14 @@ osmo_ss7_asp_find_by_socket_addr(int fd)
 				continue;
 
 			for (i = 0; i < asp->cfg.local.host_cnt; i++) {
-				bool is_any = !asp->cfg.local.host[i] || !strcmp(asp->cfg.local.host[i], "0.0.0.0");
-				if (is_any || !strcmp(asp->cfg.local.host[i], hostbuf_l))
+				bool iter_is_v6 = osmo_ip_str_type(asp->cfg.local.host[i]) == AF_INET6;
+				bool iter_is_anyaddr = host_is_ip_anyaddr(asp->cfg.local.host[i], iter_is_v6);
+				/* "::" (v6) covers "0.0.0.0" (v4), but not otherwise */
+				if (iter_is_v6 != loc_is_v6 &&
+				    !(iter_is_v6 && iter_is_anyaddr))
+					continue;
+				if (iter_is_anyaddr ||
+				    !strcmp(asp->cfg.local.host[i], hostbuf_l))
 					break;
 			}
 			if (i == asp->cfg.local.host_cnt)
@@ -1254,7 +1310,14 @@ osmo_ss7_asp_find_by_socket_addr(int fd)
 			/* If no remote host was set, it's probably a server and hence we match any cli src */
 			if (asp->cfg.remote.host_cnt) {
 				for (i = 0; i < asp->cfg.remote.host_cnt; i++) {
-					if (!asp->cfg.remote.host[i] || !strcmp(asp->cfg.remote.host[i], hostbuf_r))
+					bool iter_is_v6 = osmo_ip_str_type(asp->cfg.remote.host[i]) == AF_INET6;
+					bool iter_is_anyaddr = host_is_ip_anyaddr(asp->cfg.remote.host[i], iter_is_v6);
+					/* "::" (v6) covers "0.0.0.0" (v4), but not otherwise */
+					if (iter_is_v6 != rem_is_v6 &&
+					    !(iter_is_v6 && iter_is_anyaddr))
+						continue;
+					if (iter_is_anyaddr ||
+					    !strcmp(asp->cfg.remote.host[i], hostbuf_r))
 						break;
 				}
 				if (i == asp->cfg.remote.host_cnt)
@@ -2202,6 +2265,19 @@ int ss7_asp_rx_unknown(struct osmo_ss7_asp *asp, int ppid_mux, struct msgb *msg)
 	return 0;
 }
 
+/*! Get the logging subsystem for a given ASP. Used by generic code. */
+int osmo_ss7_asp_get_log_subsys(const struct osmo_ss7_asp *asp)
+{
+	switch (asp->cfg.proto) {
+	case OSMO_SS7_ASP_PROT_M3UA:
+		return DLM3UA;
+	case OSMO_SS7_ASP_PROT_SUA:
+		return DLSUA;
+	default:
+		return DLSS7;
+	}
+}
+
 /*! Register a call-back function for unknown SCTP PPID / IPA Stream ID */
 void osmo_ss7_register_rx_unknown_cb(osmo_ss7_asp_rx_unknown_cb *cb)
 {
