@@ -109,7 +109,7 @@ int osmo_isup_party_encode(struct msgb *msg, const char *in_digits)
 	return num_octets;
 }
 
-/*! \brief Parse wire-encoded SCCP address into omso_sccp_addr
+/*! \brief Parse wire-encoded SCCP address into osmo_sccp_addr
  *  \param[out] out user-allocated output data structure
  *  \param[in] addr wire-encoded SCCP address
  *  \param[in] addrlen Size of \ref addr in bytes
@@ -705,41 +705,54 @@ static struct xua_msg *sccp_to_xua_opt(struct msgb *msg, uint8_t *ptr_opt, struc
 	/* some bounds checking */
 	if (ptr_opt < msg->data || ptr_opt > msg->tail)
 		return NULL;
+
+	/* Q.713 section 2.3 "Coding of pointers": pointer value all zeros used
+	  to indicate that no optional param is present. */
+	if (*ptr_opt == 0)
+		return xua;
+
 	opt_start = ptr_opt + *ptr_opt;
 	if (opt_start > msg->tail)
 		return NULL;
 
 	oneopt = opt_start;
 
+	enum sccp_parameter_name_codes opt_type = 0; /* dummy value not used */
 	while (oneopt < msg->tail) {
-		uint8_t opt_type = oneopt[0];
+		uint8_t opt_len;
+		uint16_t opt_len16;
+		opt_type = oneopt[0];
 
-		if (opt_type == SCCP_PNC_END_OF_OPTIONAL)
+		switch (opt_type) {
+		case SCCP_PNC_END_OF_OPTIONAL:
 			return xua;
-
-		if (opt_type == SCCP_PNC_LONG_DATA) {
-			uint16_t opt_len16;
+		case SCCP_PNC_LONG_DATA:
 			/* two byte length field */
 			if (oneopt + 2 > msg->tail)
-				return NULL;
+				goto malformed;
 			opt_len16 = oneopt[1] << 8 | oneopt[2];
 			if (oneopt + 3 + opt_len16 > msg->tail)
-				return NULL;
+				goto malformed;
 			xua_msg_add_sccp_opt(xua, opt_type, opt_len16, oneopt+3);
 			oneopt += 3 + opt_len16;
-		} else {
-			uint8_t opt_len;
+			break;
+		default:
 			/* one byte length field */
 			if (oneopt + 1 > msg->tail)
-				return NULL;
+				goto malformed;
 
 			opt_len = oneopt[1];
 			if (oneopt + 2 + opt_len > msg->tail)
-				return NULL;
+				goto malformed;
 			xua_msg_add_sccp_opt(xua, opt_type, opt_len, oneopt+2);
 			oneopt += 2 + opt_len;
 		}
 	}
+	LOGP(DLSUA, LOGL_ERROR, "Parameter %s not found\n", osmo_sccp_pnc_name(SCCP_PNC_END_OF_OPTIONAL));
+	return NULL;
+
+malformed:
+	LOGP(DLSUA, LOGL_ERROR, "Malformed parameter %s (%d)\n", osmo_sccp_pnc_name(opt_type), opt_type);
 	return NULL;
 }
 
@@ -998,6 +1011,7 @@ static void store_local_ref(struct sccp_source_reference *local_ref, struct xua_
 	local_ref->octet3 = tmp32 & 0xff;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cr(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_request *req = (struct sccp_connection_request *)msg->l2h;
@@ -1013,6 +1027,7 @@ static struct xua_msg *sccp_to_xua_cr(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &req->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cr(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_request *req;
@@ -1029,6 +1044,7 @@ static int sua_to_sccp_cr(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &req->optional_start, req->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_confirm *cnf = (struct sccp_connection_confirm *)msg->l2h;
@@ -1041,6 +1057,7 @@ static struct xua_msg *sccp_to_xua_cc(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &cnf->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_confirm *cnf;
@@ -1055,6 +1072,7 @@ static int sua_to_sccp_cc(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &cnf->optional_start, cnf->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_cref(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_refused *ref = (struct sccp_connection_refused *)msg->l2h;
@@ -1066,6 +1084,7 @@ static struct xua_msg *sccp_to_xua_cref(struct msgb *msg, struct xua_msg *xua)
 	return sccp_to_xua_opt(msg, &ref->optional_start, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static int sua_to_sccp_cref(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_refused *ref;
@@ -1079,6 +1098,7 @@ static int sua_to_sccp_cref(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &ref->optional_start, ref->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_rlsd(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_released *rlsd = (struct sccp_connection_released *)msg->l2h;
@@ -1106,6 +1126,7 @@ static int sua_to_sccp_rlsd(struct msgb *msg, struct xua_msg *xua)
 	return xua_ies_to_sccp_opts(msg, &rlsd->optional_start, rlsd->type, xua);
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_rlc(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_connection_release_complete *rlc;
@@ -1129,6 +1150,7 @@ static int sua_to_sccp_rlc(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_dt1(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_form1 *dt1 = (struct sccp_data_form1 *) msg->l2h;
@@ -1157,6 +1179,7 @@ static int sua_to_sccp_dt1(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_udt(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata *udt = (struct sccp_data_unitdata *)msg->l2h;
@@ -1192,6 +1215,7 @@ static int sua_to_sccp_udt(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_udts(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_unitdata_service *udts;
@@ -1228,6 +1252,7 @@ static int sua_to_sccp_udts(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_it(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_data_it *it = (struct sccp_data_it *)msg->l2h;
@@ -1261,6 +1286,7 @@ static int sua_to_sccp_it(struct msgb *msg, struct xua_msg *xua)
 	return 0;
 }
 
+/*! \returns \ref xua in case of success, NULL on error (xua not freed!) */
 static struct xua_msg *sccp_to_xua_err(struct msgb *msg, struct xua_msg *xua)
 {
 	struct sccp_proto_err *err = (struct sccp_proto_err *)msg->l2h;
@@ -1302,34 +1328,54 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	switch (msg->l2h[0]) {
 	case SCCP_MSG_TYPE_CR:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_CORE);
-		return sccp_to_xua_cr(msg, xua);
+		if (!sccp_to_xua_cr(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_CC:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COAK);
-		return sccp_to_xua_cc(msg, xua);
+		if (!sccp_to_xua_cc(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_CREF:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COREF);
-		return sccp_to_xua_cref(msg, xua);
+		if (!sccp_to_xua_cref(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_RLSD:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_RELRE);
-		return sccp_to_xua_rlsd(msg, xua);
+		if (!sccp_to_xua_rlsd(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_RLC:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_RELCO);
-		return sccp_to_xua_rlc(msg, xua);
+		if (!sccp_to_xua_rlc(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_DT1:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_CODT);
-		return sccp_to_xua_dt1(msg, xua);
+		if (!sccp_to_xua_dt1(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_UDT:
 		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDT);
-		return sccp_to_xua_udt(msg, xua);
+		if (!sccp_to_xua_udt(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_UDTS:
 		xua->hdr = XUA_HDR(SUA_MSGC_CL, SUA_CL_CLDR);
-		return sccp_to_xua_udts(msg, xua);
+		if (!sccp_to_xua_udts(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_IT:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COIT);
-		return sccp_to_xua_it(msg, xua);
+		if (!sccp_to_xua_it(msg, xua))
+			goto malformed;
+		return xua;
 	case SCCP_MSG_TYPE_ERR:
 		xua->hdr = XUA_HDR(SUA_MSGC_CO, SUA_CO_COERR);
-		return sccp_to_xua_err(msg, xua);
+		if (!sccp_to_xua_err(msg, xua))
+			goto malformed;
+		return xua;
 	/* Unsupported Message Types */
 	case SCCP_MSG_TYPE_DT2:
 	case SCCP_MSG_TYPE_AK:
@@ -1341,6 +1387,10 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	case SCCP_MSG_TYPE_XUDTS:
 	case SCCP_MSG_TYPE_LUDT:
 	case SCCP_MSG_TYPE_LUDTS:
+		LOGP(DLSUA, LOGL_ERROR, "Unsupported SCCP message %s\n",
+			osmo_sccp_msg_type_name(msg->l2h[0]));
+		xua_msg_free(xua);
+		return NULL;
 	default:
 		LOGP(DLSUA, LOGL_ERROR, "Unsupported SCCP message type %u\n",
 			msg->l2h[0]);
@@ -1349,6 +1399,12 @@ struct xua_msg *osmo_sccp_to_xua(struct msgb *msg)
 	}
 
 	return NULL;
+
+malformed:
+	LOGP(DLSUA, LOGL_ERROR, "Malformed SCCP message %s\n",
+	     osmo_sccp_msg_type_name(msg->l2h[0]));
+	xua_msg_free(xua);
+	return NULL;
 }
 
 /*! \brief convert parsed SUA message to SCCP message
@@ -1412,8 +1468,11 @@ struct msgb *osmo_sua_to_sccp(struct xua_msg *xua)
 		goto out_err;
 	}
 
-	if (rc < 0)
+	if (rc < 0)  {
+		LOGP(DLSUA, LOGL_ERROR, "Malformed SUA message %s\n",
+			xua_hdr_dump(xua, &xua_dialect_sua));
 		goto out_err;
+	}
 
 	return msg;
 
