@@ -25,6 +25,7 @@
 
 #include <stdbool.h>
 #include <string.h>
+#include <limits.h>
 
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/logging.h>
@@ -39,6 +40,8 @@
 #include "xua_internal.h"
 
 /*! \brief Find a SCCP User registered for given PC+SSN or SSN only
+ * First search all users with a valid PC for a full PC+SSN match.
+ * If no such match was found, search all users with an invalid PC for an SSN-only match.
  *  \param[in] inst SCCP Instance in which to search
  *  \param[in] ssn Sub-System Number to search for
  *  \param[in] pc Point Code to search for
@@ -48,10 +51,12 @@ sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc)
 {
 	struct osmo_sccp_user *scu;
 
-	/* First try to find match for PC + SSN */
-	llist_for_each_entry(scu, &inst->users, list) {
-		if (osmo_ss7_pc_is_valid(scu->pc) && scu->pc == pc && scu->ssn == ssn)
-			return scu;
+	if (osmo_ss7_pc_is_valid(pc)) {
+		/* First try to find match for PC + SSN */
+		llist_for_each_entry(scu, &inst->users, list) {
+			if (osmo_ss7_pc_is_valid(scu->pc) && scu->pc == pc && scu->ssn == ssn)
+				return scu;
+		}
 	}
 
 	/* Then try to match on SSN only */
@@ -63,6 +68,20 @@ sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc)
 	return NULL;
 }
 
+/*! Find a SCCP User registered for given PC+SSN or SSN only.
+ * First search all users with a valid PC for a full PC+SSN match.
+ * If no match was found, search all users with an invalid PC for an SSN-only match.
+ *  \param[in] inst SCCP Instance in which to search.
+ *  \param[in] ssn Sub-System Number to search for.
+ *  \param[in] pc Point Code to search for.
+ *  \returns Matching SCCP User; NULL if none found.
+ */
+struct osmo_sccp_user *
+osmo_sccp_user_find(struct osmo_sccp_instance *inst, uint16_t ssn, uint32_t pc)
+{
+	return sccp_user_find(inst, ssn, pc);
+}
+
 /*! \brief Bind a SCCP User to a given Point Code
  *  \param[in] inst SCCP Instance
  *  \param[in] name human-readable name
@@ -171,9 +190,16 @@ static int mtp_user_prim_cb(struct osmo_prim_hdr *oph, void *ctx)
 	case OSMO_PRIM(OSMO_MTP_PRIM_TRANSFER, PRIM_OP_INDICATION):
 		/* Convert from SCCP to SUA in xua_msg format */
 		xua = osmo_sccp_to_xua(oph->msg);
+		if (!xua) {
+			LOGP(DLSCCP, LOGL_ERROR, "Couldn't convert SCCP to SUA: %s\n",
+				msgb_hexdump(oph->msg));
+			rc = -1;
+			break;
+		}
 		xua->mtp = omp->u.transfer;
 		/* hand this primitive into SCCP via the SCRC code */
 		rc = scrc_rx_mtp_xfer_ind_xua(inst, xua);
+		xua_msg_free(xua);
 		break;
 	default:
 		LOGP(DLSCCP, LOGL_ERROR, "Unknown primitive %u:%u receivd\n",
@@ -194,6 +220,7 @@ struct osmo_sccp_instance *
 osmo_sccp_instance_create(struct osmo_ss7_instance *ss7, void *priv)
 {
 	struct osmo_sccp_instance *inst;
+	int rc;
 
 	inst = talloc_zero(ss7, struct osmo_sccp_instance);
 	if (!inst)
@@ -209,6 +236,12 @@ osmo_sccp_instance_create(struct osmo_ss7_instance *ss7, void *priv)
 	inst->ss7_user.prim_cb = mtp_user_prim_cb;
 	inst->ss7_user.priv = inst;
 
+	rc = sccp_scmg_init(inst);
+	if (rc < 0) {
+		talloc_free(inst);
+		return NULL;
+	}
+
 	osmo_ss7_user_register(ss7, MTP_SI_SCCP, &inst->ss7_user);
 
 	llist_add_tail(&inst->list, &sccp_instances);
@@ -316,8 +349,8 @@ int osmo_sccp_gt_cmp(const struct osmo_sccp_gt *a, const struct osmo_sccp_gt *b)
  * The SCCP addresses' Routing Indicator is not compared, see osmo_sccp_addr_ri_cmp().
  * \param[in] a  left side.
  * \param[in] b  right side.
- * \param[in] presence_criteria  A bitmask of OSMO_SCCP_ADDR_T_* values, or 0xffffffff to compare all parts, except the
- *                               routing indicator.
+ * \param[in] presence_criteria  A bitmask of OSMO_SCCP_ADDR_T_* values, or OSMO_SCCP_ADDR_T_MASK to compare all parts,
+ *                               except the routing indicator.
  * \return -1 if a < b, 1 if a > b, and 0 if all checked values match.
  */
 int osmo_sccp_addr_cmp(const struct osmo_sccp_addr *a, const struct osmo_sccp_addr *b, uint32_t presence_criteria)
@@ -549,13 +582,33 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 	 * we intend to use. */
 	asp = osmo_ss7_asp_find_by_proto(as, prot);
 	if (!asp) {
-		/* Check if the user has already created an ASP elsewhere under
-		 * the default asp name. */
-		asp_name = talloc_asprintf(ctx, "asp-clnt-%s", name);
-		asp = osmo_ss7_asp_find_by_name(ss7, asp_name);
+		/* Check if the user has created an ASP for this proto that is not added on any AS yet. */
+		struct osmo_ss7_asp *asp_i;
+		llist_for_each_entry(asp_i, &ss7->asp_list, list) {
+			struct osmo_ss7_as *as_i;
+			bool is_on_as = false;
+			if (asp_i->cfg.proto != prot)
+				continue;
+			llist_for_each_entry(as_i, &ss7->as_list, list) {
+				if (!osmo_ss7_as_has_asp(as_i, asp_i))
+					continue;
+				is_on_as = true;
+				break;
+			}
+			if (is_on_as) {
+				/* This ASP is already on another AS. If it was on this AS, we'd have found it above. */
+				continue;
+			}
+			/* This ASP matches the protocol and is not yet associated to any AS. Use it. */
+			asp = asp_i;
+			LOGP(DLSCCP, LOGL_NOTICE, "%s: ASP %s for %s is not associated with any AS, using it\n",
+			     name, asp->cfg.name, osmo_ss7_asp_protocol_name(prot));
+			break;
+		}
 		if (!asp) {
-			LOGP(DLSCCP, LOGL_NOTICE, "%s: Creating ASP instance\n",
-			     name);
+			asp_name = talloc_asprintf(ctx, "asp-clnt-%s", name);
+			LOGP(DLSCCP, LOGL_NOTICE, "%s: No unassociated ASP for %s, creating new ASP %s\n",
+			     name, osmo_ss7_asp_protocol_name(prot), asp_name);
 			asp =
 			    osmo_ss7_asp_find_or_create(ss7, asp_name,
 							default_remote_port,
@@ -565,22 +618,10 @@ osmo_sccp_simple_client_on_ss7_id(void *ctx, uint32_t ss7_id, const char *name,
 			if (!asp)
 				goto out_rt;
 			asp_created = true;
-
-			asp->cfg.local.host[0] = NULL;
-			asp->cfg.remote.host[0] = NULL;
-			if (default_local_ip) {
-				asp->cfg.local.host[0] =
-				    talloc_strdup(asp, default_local_ip);
-			}
-			if (default_remote_ip) {
-				asp->cfg.remote.host[0] =
-				    talloc_strdup(asp, default_remote_ip);
-			}
-			asp->cfg.local.host_cnt = 1;
-			asp->cfg.remote.host_cnt = 1;
+			osmo_ss7_asp_peer_set_hosts(&asp->cfg.local, asp, &default_local_ip, 1);
+			osmo_ss7_asp_peer_set_hosts(&asp->cfg.remote, asp, &default_remote_ip, 1);
 			asp->simple_client_allocated = true;
-		} else
-			talloc_free(asp_name);
+		}
 
 		osmo_ss7_as_add_asp(as, asp->cfg.name);
 	}
@@ -695,7 +736,7 @@ osmo_sccp_simple_server(void *ctx, uint32_t pc,
 			enum osmo_ss7_asp_protocol prot, int local_port,
 			const char *local_ip)
 {
-	return osmo_sccp_simple_server_on_ss7_id(ctx, 1, pc, prot,
+	return osmo_sccp_simple_server_on_ss7_id(ctx, 0, pc, prot,
 					         local_port, local_ip);
 }
 
@@ -710,6 +751,7 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 	struct osmo_ss7_as *as;
 	struct osmo_ss7_route *rt;
 	struct osmo_ss7_asp *asp;
+	struct osmo_xua_server *oxs;
 	char *as_name, *asp_name;
 
 	if (local_port < 0)
@@ -734,6 +776,15 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 	asp = osmo_ss7_asp_find_or_create(ss7, asp_name, remote_port, local_port, prot);
 	if (!asp)
 		goto out_rt;
+	oxs = osmo_ss7_xua_server_find(ss7, prot, local_port);
+	if (!oxs)
+		goto out_asp;
+	if (osmo_ss7_asp_peer_set_hosts(&asp->cfg.local, asp,
+					(const char* const*)oxs->cfg.local.host,
+					oxs->cfg.local.host_cnt) < 0)
+		goto out_asp;
+	if (osmo_ss7_asp_peer_add_host(&asp->cfg.remote, asp, remote_ip) < 0)
+		goto out_asp;
 	asp->cfg.is_server = true;
 	asp->cfg.role = OSMO_SS7_ASP_ROLE_SG;
 	osmo_ss7_as_add_asp(as, asp_name);
@@ -743,6 +794,8 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 
 	return ss7->sccp;
 
+out_asp:
+	osmo_ss7_asp_destroy(asp);
 out_rt:
 	osmo_ss7_route_destroy(rt);
 out_as:
