@@ -34,18 +34,13 @@
 
 /* The general idea is:
  * * translate incoming SUA/M3UA msg_class/msg_type to xua_asp_event
- * * propagate state transitions to XUA_AS_FSM via _onenter functiosn
+ * * propagate state transitions to XUA_AS_FSM via _onenter functions
  * * notify the Layer Management of any relevant changes
  * *
  */
 
 /* According to RFC3868 Section 8 */
-#define XUA_T_A_SEC	2
-#define XUA_T_R_SEC	2
 #define XUA_T_ACK_SEC	2
-#define XUA_T_BEAT_SEC	30
-#define SUA_T_IAS_SEC	(7*60)		/* SUA only */
-#define SUA_T_IAR_SEC	(15*60)		/* SUA only */
 
 static const struct value_string xua_asp_event_names[] = {
 	{ XUA_ASP_E_M_ASP_UP_REQ,	"M-ASP_UP.req" },
